/*
 * lm2latex - src/lm2latex.c
 * Copyright (C) 2010 Sebastian Harl <sh@tokkee.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * A Linux-Magazin markup to LaTeX converter.
 */

#if HAVE_CONFIG_H
#	include "config.h"
#endif /* HAVE_CONFIG_H */

#include "l2l.h"
#include "l2l_features.h"

#include <errno.h>

#if HAVE_LIBGEN_H
#	include <libgen.h>
#else /* HAVE_LIBGEN_H */
#	define basename(path) (path)
#endif /* ! HAVE_LIBGEN_H */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <unistd.h>

static void
exit_usage(char *name, int status)
{
	printf(
"Usage: %s <options> <infile> [<outfile>]\n"

"\nOptions:\n"
"  -h    display this help and exit\n"
"  -V    display the version number and copyright\n"

"\nlm2latex "L2L_VERSION_STRING L2L_VERSION_EXTRA", "PACKAGE_URL"\n",
basename(name));
	exit(status);
} /* exit_usage */

static void
exit_version(void)
{
	printf("lm2latex version "L2L_VERSION_STRING L2L_VERSION_EXTRA", "
			"built "BUILD_DATE"\n"
			"Copyright (C) 2010 "PACKAGE_BUGREPORT"\n"

			"\nThis is free software under the terms of the BSD license, see "
			"the source for\ncopying conditions. There is NO WARRANTY; not "
			"even for MERCHANTABILITY or\nFITNESS FOR A PARTICULAR "
			"PURPOSE.\n");
	exit(0);
} /* exit_version */

int
main(int argc, char **argv)
{
	char *infile = NULL, *outfile = NULL;
	FILE *in_fh, *out_fh;
	int   args;

	while (42) {
		int opt = getopt(argc, argv, "hV");

		if (-1 == opt)
			break;

		switch (opt) {
			case 'h':
				exit_usage(argv[0], 0);
				break;
			case 'V':
				exit_version();
				break;
			default:
				exit_usage(argv[0], 1);
		}
	}

	args = argc - optind;
	if ((0 < args) && (args <= 2)) {
		infile = argv[optind];

		if (1 < args)
			outfile = argv[optind + 1];
		else
			outfile = "-";
	}
	else
		exit_usage(argv[0], 1);

	if (! strcmp(infile, "-"))
		in_fh = stdin;
	else
		in_fh = fopen(infile, "r");

	if (! in_fh) {
		fprintf(stderr, "lm2latex: FATAL: Failed to open ``%s'' "
				"for reading: %s\n", infile, strerror(errno));
		return 1;
	}

	if (! strcmp(outfile, "-"))
		out_fh = stdout;
	else
		out_fh = fopen(outfile, "w");

	if (! out_fh) {
		fprintf(stderr, "lm2latex: FATAL: Failed to open ``%s'' "
				"for writing: %s\n", outfile, strerror(errno));
		return 1;
	}

	l2l_convert_file_fh(in_fh, out_fh);

	if (in_fh != stdin)
		fclose(in_fh);
	if (out_fh != stdout)
		fclose(out_fh);
	return 0;
} /* main */

/* vim: set tw=78 sw=4 ts=4 noexpandtab : */

